import {Component, ViewChild} from '@angular/core'
import {ActivatedRoute, Router} from '@angular/router'
import {EmployeeGroup, EmployeePermission} from '../../../shared/model/employee'
import {GroupService} from '../../services/group.service'
import {FormBuilder, FormControl, FormGroup, Validators} from '@angular/forms'
import {PermissionsFieldComponent} from '../../../shared/components/permissions-field/permissions-field.component'
import {AccountService} from '../../../accounts/services/account.service'
import {ErrorHandlingComponent} from '../../../shared/components/subscribing.component'
import {ErrorService} from '../../../shared/service/error.service'

@Component({
  selector: 'cre-edit',
  templateUrl: './edit.component.html',
  styleUrls: ['./edit.component.sass']
})
export class EditComponent extends ErrorHandlingComponent {
  @ViewChild('permissionsField') permissionsField: PermissionsFieldComponent

  group: EmployeeGroup | null

  private _nameControl: FormControl

  constructor(
    private accountService: AccountService,
    private groupService: GroupService,
    private formBuilder: FormBuilder,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
  }

  ngOnInit(): void {
    const id = parseInt(this.activatedRoute.snapshot.paramMap.get('id'))
    this.subscribeEntityById(
      this.groupService,
      id,
      group => this.group = group,
      '/group/list'
    )
  }

  submit(): void {
    if (this.form.valid && this.permissionsField.valid()) {
      this.subscribeAndNavigate(
        this.groupService.update(this.group.id, this.nameControl.value, this.permissionsField.allEnabledPermissions),
        '/group/list'
      )
    }
  }

  delete() {
    this.subscribeAndNavigate(
      this.groupService.delete(this.group.id),
      '/group/list'
    )
  }

  get form(): FormGroup {
    return this.formBuilder.group({
      name: this.nameControl
    })
  }

  get confirmBoxMessage(): string {
    return `Voulez-vous vraiment supprimer le groupe ${this.group.name}?`
  }

  get nameControl(): FormControl {
    if (this._nameControl) {
      return this._nameControl
    }
    if (this.group) {
      this._nameControl = new FormControl(this.group.name, Validators.compose([Validators.required, Validators.minLength(3)]))
      return this._nameControl
    }
    return null
  }

  get canRemoveGroup(): boolean {
    return this.accountService.hasPermission(EmployeePermission.REMOVE_EMPLOYEE_GROUP)
  }
}
